// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of matchers;

class _EmptyIterable implements Matcher<Iterable> {
  bool matches(Iterable iterable) => iterable.isEmpty;
  String get description => "be empty";
}

class _IterableContains<T> implements Matcher<Iterable<T>> {
  T obj;

  _IterableContains(this.obj);

  bool matches(Iterable<T> iterable) => iterable.contains(obj);
  String get description => "contain '$obj'";
}

class _EveryElement<T> implements Matcher<Iterable<T>> {
  Matcher<T> _matcher;

  _EveryElement(this._matcher);

  bool matches(Iterable<T> iterable) => iterable.every((e) => _matcher.matches(e));
  String get description => "every element ${_matcher.description}";
}

class _AnyElement<T> implements Matcher<Iterable<T>> {
  Matcher<T> _matcher;

  _AnyElement(this._matcher);

  bool matches(Iterable<T> iterable) => iterable.any((e) => _matcher.matches(e));
  String get description => "any element ${_matcher.description}";
}

class _IterableLength implements Matcher<Iterable> {
  Matcher<int> _matcher;

  _IterableLength(this._matcher);

  bool matches(Iterable iterable) => _matcher.matches(iterable.length);
  String get description => "iterable length ${_matcher.description}";
}

class _OrderedEquals<T> implements Matcher<Iterable<T>>, HasRichMessage<Iterable<T>> {
  List<T> _expected;

  _OrderedEquals(this._expected);

  bool matches(Iterable<T> actualIterable) {
    List<T> actual = new List<T>.from(actualIterable);

    if (_expected.length != actual.length) {
      return false;
    }

    for (int i = 0; i < _expected.length; i++) {
      if (_expected[i] != actual[i]) {
        return false;
      }
    }

    return true;
  }

  String get description => "be equal to $_expected (ordered)";

  String buildMessage(Iterable<T> actual)
      => _diffLists(new List<T>.from(_expected), new List<T>.from(actual));
}

class _UnorderedEquals<T> implements Matcher<Iterable<T>> {
  Set<T> _expected;

  _UnorderedEquals(this._expected);

  bool matches(Iterable<T> actualIterable) {
    Set<T> actual = new Set<T>.from(actualIterable);

    Set<T> missing = new Set<T>.from(_expected);
    missing.removeAll(actual);
    Set<T> extra = new Set<T>.from(actual);
    extra.removeAll(_expected);

    return missing.isEmpty && extra.isEmpty;
  }

  String get description => "be equal to $_expected (unordered)";
}

/// Every iterable that is empty (i.e., `it.isEmtpy`) matches.
Matcher<Iterable> emptyIterable() => new _EmptyIterable();

/// Every iterable that contains given [obj] (i.e., `it.contains(obj)`)
/// matches.
Matcher<Iterable/*<T>*/> iterableContains(/*T*/ obj) => new _IterableContains/*<T>*/(obj);

/// Every iterable in which all elements match given [matcher] matches.
Matcher<Iterable/*<T>*/> everyElement(Matcher/*<T>*/ matcher) => new _EveryElement/*<T>*/(matcher);

/// Every iterable in which at least one element matches given [matcher]
/// matches.
Matcher<Iterable/*<T>*/> anyElement(Matcher/*<T>*/ matcher) => new _AnyElement/*<T>*/(matcher);

/// Every iterable whose length matches given [matcher] matches.
Matcher<Iterable> iterableLength(Matcher<int> matcher) => new _IterableLength(matcher);

/// Every iterable which has the same length as [expected] iterable and
/// contains all elements of the [expected] iterable in the same order
/// matches. Note that both the iterables (expected and actual) can be
/// iterated many times!
Matcher<Iterable/*<T>*/> orderedEquals(Iterable/*<T>*/ expected) => new _OrderedEquals(new List/*<T>*/.from(expected));

/// Every iterable which has the same length as [expected] iterable and
/// contains all elements of the [expected] iterable, regardless of the order,
/// matches. Note that both the iterables (expected and actual) can be
/// iterated many times!
Matcher<Iterable/*<T>*/> unorderedEquals(Iterable/*<T>*/ expected) => new _UnorderedEquals(new Set/*<T>*/.from(expected));
